
#ifdef __BORLANDC__
	// Borland stuff.
	#include <windows.h>
	#pragma hdrstop
#endif


// Primary include.
#include "f00BoneSystem.h"

// System includes.
#include <math.h>

// Local includes.
#include "f00PI.h"
#include "f00TextureObject.h"



f00BoneSystem::f00BoneSystem()
:	f00QuatObject(),
	d_showBonesAsLines( false ),
	d_showWireframe( false )
{
}


f00BoneSystem::~f00BoneSystem()
{
}


void
f00BoneSystem::renderObject( const f00Point3D& viewPos, const f00Quat& viewDir, const f00TimeBase& timeBase )
{
	// Smack the base object.
	f00QuatObject::renderObject( viewPos, viewDir, timeBase );

	// Draw the bones in overlay mode?
	if( d_showBonesAsLines )
	{
		// FIXME Use attrib stack.
//		glDisable( GL_DEPTH_TEST );
		glDepthFunc( GL_ALWAYS );
		glDisable( GL_LIGHTING );
		glDepthRange( 0.0, 0.0 );
		glColor3f( 1.0, 0.5, 0.5 );
		glBegin( GL_LINE_STRIP );
			f00Quat curOrient;
			f00Point3D curEnd( 0.0, 0.0, 0.0 );
			glVertex3fv( curEnd.getArray() );
			for( int bone = 0; bone < (int)d_boneList.size(); bone++ )
			{
				BoneInfo& bI( *( d_boneList[ bone ] ) );
				curOrient.postMult( bI.d_orientation );
				f00Point3D localEnd( curOrient.rotatePoint( f00Point3D( 0.0, 0.0, bI.d_length ) ) );
				curEnd += localEnd;
				glVertex3fv( curEnd.getArray() );
			}
//glVertex3f( 0.0, 0.0, 20.0 );
		glEnd();
		glColor3f( 1.0, 1.0, 1.0 );
		glDepthRange( 0.0, 1.0 );
		glEnable( GL_LIGHTING );
		glDepthFunc( GL_LESS );
//		glEnable( GL_DEPTH_TEST );
	}

	// Wireframe?
	if( d_showWireframe )
	{
		glPolygonMode( GL_FRONT_AND_BACK, GL_LINE );
		glDisable( GL_CULL_FACE );
	}
	else
	{
		glPolygonMode( GL_FRONT_AND_BACK, GL_FILL );
		glEnable( GL_CULL_FACE );
	}
}


void
f00BoneSystem::addBone( f00BoneSystem::BoneInfo* pBI )
{
	if( pBI == 0 )
		return;
	d_boneList.push_back( pBI );
}


int
f00BoneSystem::countBones()
{
	return (int)d_boneList.size();
}


f00BoneSystem::BoneInfo*
f00BoneSystem::getBone( int index )
{
	if( ( index < 0 ) || ( index >= (int)d_boneList.size() ) )
		return 0;
	return d_boneList[ index ];
}


void
f00BoneSystem::dropBone( int index )
{
	if( ( index < 0 ) || ( index >= (int)d_boneList.size() ) )
		return;
	delete d_boneList[ index ];
	d_boneList.erase( d_boneList.begin() + index, d_boneList.begin() + index + 1 );
}



