
#ifdef __BORLANDC__
	// Borland stuff.
	#include <windows.h>
	#pragma hdrstop
#endif


// Primary include.
#include "f00Point3D.h"

// System includes.
#include <stdio.h>
#include <math.h>
#include <assert.h>



f00Point3D::f00Point3D( GLfloat* pArray )
:	d_x( d_array[ 0 ] ),
	d_y( d_array[ 1 ] ),
	d_z( d_array[ 2 ] ),
	d_w( d_array[ 3 ] )
{
	assert( pArray );
	memcpy( d_array, pArray, 4 * sizeof( GLfloat ) );
}


f00Point3D::f00Point3D( GLfloat x, GLfloat y, GLfloat z, GLfloat w )
:	d_x( d_array[ 0 ] ),
	d_y( d_array[ 1 ] ),
	d_z( d_array[ 2 ] ),
	d_w( d_array[ 3 ] )
{
	set( x, y, z, w );
}


f00Point3D::f00Point3D( const f00Point3D& copyFrom )
:	d_x( d_array[ 0 ] ),
	d_y( d_array[ 1 ] ),
	d_z( d_array[ 2 ] ),
	d_w( d_array[ 3 ] )
{
	*this = copyFrom;
}


f00Point3D&
f00Point3D::operator = ( const f00Point3D& copyFrom )
{
	memcpy( d_array, copyFrom.d_array, 4 * sizeof( GLfloat ) );
	return *this;
}


f00Point3D
f00Point3D::operator + ( const f00Point3D& addOn ) const
{
	f00Point3D retVal( *this );
	return retVal += addOn;
}


f00Point3D&
f00Point3D::operator += ( const f00Point3D& addOn )
{
	d_x += addOn.d_x;
	d_y += addOn.d_y;
	d_z += addOn.d_z;
	d_w += addOn.d_w;
	return *this;
}


f00Point3D
f00Point3D::operator - ( const f00Point3D& addOn ) const
{
	f00Point3D retVal( *this );
	return retVal -= addOn;
}


f00Point3D&
f00Point3D::operator -= ( const f00Point3D& addOn )
{
	d_x -= addOn.d_x;
	d_y -= addOn.d_y;
	d_z -= addOn.d_z;
	d_w -= addOn.d_w;
	return *this;
}


f00Point3D
f00Point3D::operator * ( float scalar ) const
{
	f00Point3D retVal( *this );
	return retVal *= scalar;
}


f00Point3D&
f00Point3D::operator *= ( float scalar )
{
	d_x *= scalar;
	d_y *= scalar;
	d_z *= scalar;
	d_w *= scalar;
	return *this;
}


GLdouble
f00Point3D::length( const f00Point3D& pointA )
{
	return( sqrt( pointA.d_x * pointA.d_x + pointA.d_y * pointA.d_y + pointA.d_z * pointA.d_z ) );
}


GLdouble
f00Point3D::length() const
{
	return length( *this );
}


void
f00Point3D::normalize( f00Point3D& pointA )
{
	GLdouble radius = pointA.length();
	if( radius == 0. )
	{
		pointA.d_z = 1.0;		// Just set it arbitratily.
		return;
	}

	pointA.d_x /= radius;
	pointA.d_y /= radius;
	pointA.d_z /= radius;
}


void
f00Point3D::normalize()
{
	normalize( *this );
}


// Result is non-normalized!
f00Point3D
f00Point3D::crossProduct( const f00Point3D& pointA, const f00Point3D& pointB )
{
	f00Point3D retVal;
	retVal.d_x = pointA.d_y * pointB.d_z - pointB.d_y * pointA.d_z;
	retVal.d_y = pointA.d_z * pointB.d_x - pointB.d_z * pointA.d_x;
	retVal.d_z = pointA.d_x * pointB.d_y - pointB.d_x * pointA.d_y;
	return retVal;
}


f00Point3D
f00Point3D::crossProduct( const f00Point3D& crossWith ) const
{
	return crossProduct( *this, crossWith );
}


// Not the most efficient, try not to overuse.
f00Point3D
f00Point3D::normalFrom3PointPlane( const f00Point3D& pointA, const f00Point3D& pointB, const f00Point3D& pointC )
{
	GLfloat xDelta1( pointB.d_x - pointA.d_x );
	GLfloat yDelta1( pointB.d_y - pointA.d_y );
	GLfloat zDelta1( pointB.d_z - pointA.d_z );

	GLfloat xDelta2( pointC.d_x - pointA.d_x );
	GLfloat yDelta2( pointC.d_y - pointA.d_y );
	GLfloat zDelta2( pointC.d_z - pointA.d_z );

	f00Point3D retVal;
	retVal.d_x = yDelta1 * zDelta2 - yDelta2 * zDelta1;
	retVal.d_y = zDelta1 * xDelta2 - zDelta2 * xDelta1;
	retVal.d_z = xDelta1 * yDelta2 - xDelta2 * yDelta1;

	retVal.normalize();
	return retVal;
}



