#ifndef f00Point3DH
#define f00Point3DH


// System includes.
#include <GL/gl.h>



class f00Point3D
{

	// Constructors, destructor.
	public:

		// Must point to an array of 4 or more GLfloats: x, y, z, w
		f00Point3D( GLfloat* pArray );

		f00Point3D( GLfloat x = 0.0, GLfloat y = 0.0, GLfloat z = 0.0, GLfloat w = 1.0 );

		f00Point3D( const f00Point3D& copyFrom );


	// Accessors, manipulators.
	public:

		f00Point3D&
		operator = ( const f00Point3D& copyFrom );

		f00Point3D
		operator + ( const f00Point3D& addOn ) const;

		f00Point3D&
		operator += ( const f00Point3D& addOn );

		f00Point3D
		operator - ( const f00Point3D& addOn ) const;

		f00Point3D&
		operator -= ( const f00Point3D& addOn );

		f00Point3D
		operator * ( float scalar ) const;

		f00Point3D&
		operator *= ( float scalar );


		// Only applies to 3-space.  d_w is *not* included.
		static GLdouble
		length( const f00Point3D& pointA );

		GLdouble
		length() const;


		// Only applies to 3-space.  d_w is *not* affected.
		static void
		normalize( f00Point3D& pointA );

		void
		normalize();


		// Results are non-normalized!
		static f00Point3D
		crossProduct( const f00Point3D& pointA, const f00Point3D& pointB );

		f00Point3D
		crossProduct( const f00Point3D& crossWith ) const;


		// Ownership *stays* in f00Point3D!
		const GLfloat*
		getArray() const
		{
			return d_array;
		}


		GLfloat
		x() const
		{
			return d_x;
		}

		GLfloat
		y() const
		{
			return d_y;
		}

		GLfloat
		z() const
		{
			return d_z;
		}

		GLfloat
		w() const
		{
			return d_w;
		}


		GLfloat&
		xRef()
		{
			return d_x;
		}

		GLfloat&
		yRef()
		{
			return d_y;
		}

		GLfloat&
		zRef()
		{
			return d_z;
		}

		GLfloat&
		wRef()
		{
			return d_w;
		}


		void
		x( GLfloat x )
		{
			d_x = x;
		}

		void
		y( GLfloat y )
		{
			d_y = y;
		}

		void
		z( GLfloat z )
		{
			d_z = z;
		}

		void
		w( GLfloat w )
		{
			d_w = w;
		}


		f00Point3D&
		set( const f00Point3D& copyFrom )
		{
			*this = copyFrom;
			return *this;
		}


		f00Point3D&
		set( GLfloat x = 0.0, GLfloat y = 0.0, GLfloat z = 0.0, GLfloat w = 0.0 )
		{
			d_x = x;
			d_y = y;
			d_z = z;
			d_w = w;
			return *this;
		}


		// Not the most efficient, try not to overuse.
		// [ Uses: mults, divs and a sqrt()!!! ]
		static f00Point3D
		normalFrom3PointPlane( const f00Point3D& pointA, const f00Point3D& pointB, const f00Point3D& pointC );


	// Data members.
	private:

		GLfloat					d_array[ 4 ];

		GLfloat&					d_x;
		GLfloat&					d_y;
		GLfloat&	 				d_z;
		GLfloat&					d_w;


};


#endif

