#ifndef f00QuatH
#define f00QuatH


// System includes.
#include <GL/gl.h>

// Local includes.
#include "f00Point3D.h"



class f00Quat
{

	// Constructors, destructor.
	public:

		f00Quat();

		f00Quat( const f00Quat& copyFrom );

		f00Quat( GLfloat x, GLfloat y, GLfloat z );

		f00Quat( GLfloat angle, GLfloat x, GLfloat y, GLfloat z );


	// Accessors, manipulators.
	public:

		f00Quat&
		reset();

		f00Quat&
		set( const f00Quat& copyFrom );

		f00Quat&
		set( GLfloat x, GLfloat y, GLfloat z );

		f00Quat&
		set( GLfloat angle, GLfloat x, GLfloat y, GLfloat z );


		f00Quat&
		postMult( const f00Quat& quat );

		f00Quat&
		multAndSet( const f00Quat& quat1, const f00Quat& quat2 );


		f00Quat&
		normalize();

		// Only valid until next getMatrix call....
		GLfloat*
		getMatrix( bool doNormalize = true );

		void
		getMatrix( GLfloat matrix[ 16 ], bool doNormalize = true );

		void
		getAxisAngle( GLfloat& axisX, GLfloat& axisY, GLfloat& axisZ, GLfloat& rotAngle );

		void
		getLatLong( GLfloat& latAngle, GLfloat& longAngle );

		void
		getDirectionVector( GLfloat& dirX, GLfloat& dirY, GLfloat& dirZ );

		GLfloat
		getSquaredLen() const;

		void
		invert();

		bool
		isZero() const;

		f00Point3D
		rotatePoint( const f00Point3D& p1, bool doNormalize = true );

		GLfloat
		dotProduct( const f00Quat& q1 ) const;


	// Operators.
	public:

		f00Quat
		operator * ( const f00Quat& q1 ) const;

		f00Quat&
		operator *= ( const f00Quat& q1 );

		f00Quat
		operator + ( const f00Quat& q1 ) const;

		f00Quat&
		operator += ( const f00Quat& q1 );

		f00Quat
		operator - ( const f00Quat& q1 ) const;

		f00Quat&
		operator -= ( const f00Quat& q1 );


	// Data members.
	private:

		GLfloat d_val[ 4 ];


};


#endif

