
#ifdef __BORLANDC__
	// Borland stuff.
	#include <windows.h>
	#pragma hdrstop
#endif


// Primary include.
#include "f00QuatObject.h"

// System includes.
#include <math.h>

// Local includes.
#include "f00PI.h"



f00QuatObject::f00QuatObject(	GLfloat x, GLfloat y, GLfloat z )
:	d_x( x ),
	d_y( y ),
	d_z( z ),
	d_xDelta( 0.0 ),
	d_yDelta( 0.0 ),
	d_zDelta( 0.0 )
{
}


f00QuatObject::f00QuatObject(	GLfloat x, GLfloat y, GLfloat z, const f00Quat& rot )
:	d_x( x ),
	d_y( y ),
	d_z( z ),
	d_rot( rot )
{
}


f00QuatObject::~f00QuatObject()
{
}


void
f00QuatObject::render( const f00Point3D& viewPos, const f00Quat& viewDir, const f00TimeBase& timeBase )
{
	glPushMatrix();

	renderObject( viewPos, viewDir, timeBase );

	glPopMatrix();
}


void
f00QuatObject::resetRot()
{
	d_rot.reset();
}


void
f00QuatObject::resetRotDelta()
{
	d_rotDelta.reset();
}


void
f00QuatObject::resetPos()
{
	d_x = 0.0;
	d_y = 0.0;
	d_z = 0.0;
}


void
f00QuatObject::resetPosDelta()
{
	d_xDelta = 0.0;
	d_yDelta = 0.0;
	d_zDelta = 0.0;
}


void
f00QuatObject::pointAt( GLfloat x, GLfloat y, GLfloat z )
{
	//
	// Pick two eulers that will work.  Default 'up' as along +Y.
	//

	GLfloat xD( d_x - x );
	GLfloat yD( d_y - y );
	GLfloat zD( d_z - z );
	GLfloat angleX( 0.0 );
	GLfloat angleY( 0.0 );

	// Always reset to begin with.
	d_rot.reset();

	// First, rotate about the Y axis....
	if( ( zD != 0.0f ) || ( yD != 0.0f ) )
	{
		angleY = atan2( xD, zD ) * 180.0 / M_PI;
	}

	// Now, rotate about the x axis to align with the point.
	float lenD( sqrt( xD * xD + zD * zD + yD * yD ) );
	if( lenD != 0.0f )
	{
		angleX = asin( yD / lenD ) * 180.0 / M_PI;
	}

	// Have to do it in this order....
	d_rot.postMult( f00Quat( 0.0, -angleY, 0.0 ) );
	d_rot.postMult( f00Quat( angleX, 0.0, 0.0 ) );
}


void
f00QuatObject::alignTo( const f00QuatObject& subject )
{
	moveTo( subject );
	rotTo( subject );
}



//
// Virtuals
//


void
f00QuatObject::renderObject( const f00Point3D& viewPos, const f00Quat& viewDir, const f00TimeBase& timeBase )
{
	glTranslatef( d_x, d_y, d_z );
	glMultMatrixf( d_rot.getMatrix() );
}


void
f00QuatObject::reset()
{
	resetRot();
	resetRotDelta();
	resetPos();
	resetPosDelta();
}


void
f00QuatObject::update()
{
	moveBy( d_xDelta, d_yDelta, d_zDelta );
	rotBy( d_rotDelta );
}


void
f00QuatObject::truck( GLfloat truckBy, bool truckX, bool truckY, bool truckZ )
{
	// Build the orientation to move along.
	f00Quat truckQuat( rot() );
	if( truckZ )
		;	// Do nothing.
	else if( truckX )
		truckQuat.postMult( f00Quat( 0.0, -90.0, 0.0 ) );
	else if( truckY )
		truckQuat.postMult( f00Quat( 90.0, 0.0, 0.0 ) );

	// Ask the Quat class for a unit vector, aligned with the orientation.
	GLfloat dirX;
	GLfloat dirY;
	GLfloat dirZ;
	truckQuat.getDirectionVector( dirX, dirY, dirZ );

	// Use the values to update the position....
	moveBy( dirX * truckBy, dirY * truckBy, dirZ * truckBy );
}


void
f00QuatObject::truckVelocity( GLfloat truckBy, bool truckX, bool truckY, bool truckZ )
{
	// This saves recreating them each time.
	static f00Quat quatZ( 0.0, -90.0, 0.0 );
	static f00Quat quatY( 90.0, 0.0, 0.0 );

	// Build the orientation to move along.
	f00Quat truckQuat( rot() );
	if( truckZ )
		;	// Do nothing.
	else if( truckX )
		truckQuat.postMult( quatZ );
	else if( truckY )
		truckQuat.postMult( quatY );

	// Ask the Quat class for a unit vector, aligned with the orientation.
	GLfloat dirX;
	GLfloat dirY;
	GLfloat dirZ;
	truckQuat.getDirectionVector( dirX, dirY, dirZ );

	// Use the values to update the deltas....
	d_xDelta += dirX * truckBy;
	d_yDelta += dirY * truckBy;
	d_zDelta += dirZ * truckBy;

}



