#ifndef f00QuatObjectH
#define f00QuatObjectH


// System includes.
#include <GL/gl.h>

// Local includes.
#include "f00Quat.h"
#include "f00Point3D.h"
#include "f00TimeBase.h"



//
// Base class for renderable/positional objects.
//
// FIXME  I'll have to revisit the d_rotDelta issue later, a quat cannot store a rotation
// of >= 180 degrees without 'tumbling' to a different value.  Bugger it.
//

class f00QuatObject
{
	// Stuff.
	public:

		f00QuatObject( GLfloat x = 0.0, GLfloat y = 0.0, GLfloat z = 0.0 );

		f00QuatObject( GLfloat x, GLfloat y, GLfloat z, const f00Quat& rot );

		virtual
		~f00QuatObject();


	// General stuff.
	public:

		void
		render( const f00Point3D& viewPos, const f00Quat& viewDir, const f00TimeBase& timeBase );


		void
		resetPos();

		void
		resetPosDelta();


		void
		resetRot();

		void
		resetRotDelta();


		void
		pointAt( GLfloat x, GLfloat y, GLfloat z );

		void
		alignTo( const f00QuatObject& subject );


	// Pos accessors.
	public:

		GLfloat
		x() const						 	{ return d_x; }

		GLfloat
		y() const							{ return d_y; }

		GLfloat
		z() const						 	{ return d_z; }


		GLfloat&
		xRef()							 	{ return d_x; }

		GLfloat&
		yRef()								{ return d_y; }

		GLfloat&
		zRef()							 	{ return d_z; }


		void
		moveTo( GLfloat x, GLfloat y, GLfloat z )		{ d_x = x; d_y = y; d_z = z; }

		void
		moveTo( const f00QuatObject& subject )			{ d_x = subject.d_x; d_y = subject.d_y; d_z = subject.d_z; }

		void
		moveBy( GLfloat x, GLfloat y, GLfloat z )		{ d_x += x; d_y += y; d_z += z; }


		GLfloat
		xDelta() const						{ return d_xDelta; }

		GLfloat
		yDelta() const						{ return d_yDelta; }

		GLfloat
		zDelta() const						{ return d_zDelta; }


		void
		xDelta( GLfloat xDelta )		{ d_xDelta = xDelta; }

		void
		yDelta( GLfloat yDelta )		{ d_yDelta = yDelta; }

		void
		zDelta( GLfloat zDelta )		{ d_zDelta = zDelta; }


	// Rot accessors.
	public:

		f00Quat
		rot() const							{ return d_rot; }

		f00Quat&
		rotRef()								{ return d_rot; }

		void
		rotTo( const f00Quat& aQuat )						{ d_rot = aQuat; }

		void
		rotTo( const f00QuatObject& subject )			{ d_rot = subject.d_rot; }

		void
		rotBy( const f00Quat& aQuat )						{ d_rot.postMult( aQuat ); }


		f00Quat
		rotDelta() const					{ return d_rotDelta; }

		void
		rotDeltaTo( f00Quat& aQuat )	{ d_rotDelta = aQuat; }


	// Virtuals.
	public:

		virtual void
		renderObject( const f00Point3D& viewPos, const f00Quat& viewDir, const f00TimeBase& timeBase );

		virtual void
		reset();

		virtual void
		update();


		// This is called when we want to 'truck' the object (ie: move it orthogonally,
		// relative to it's current orientation).
		void
		truck( GLfloat truckBy, bool truckX, bool truckY, bool truckZ );

		// This is called when we want to 'truck' the object's velocity.
		void
		truckVelocity( GLfloat truckBy, bool truckX, bool truckY, bool truckZ );


	// Data members.
	private:

		GLfloat								d_x;
		GLfloat								d_y;
		GLfloat								d_z;

		GLfloat								d_xDelta;
		GLfloat								d_yDelta;
		GLfloat								d_zDelta;

		f00Quat								d_rot;
		f00Quat								d_rotDelta;		// FIXME  I know, BAD! EVIL!


};


#endif

