
#ifdef __BORLANDC__
	// Borland stuff.
	#include <windows.h>
	#pragma hdrstop
#endif


// Primary include.
#include "f00TargaLoader.h"

// System includes.
#include <assert.h>

// Local includes.
#include "f00FileLoader.h"


#define DEF_targaHeaderLength			12
#define DEF_targaHeaderContent		"\x00\x02\x00\x00\x00\x00\x00\x00\x00\x00\x00"
#define DEF_targaBodyStartsAt			DEF_targaHeaderLength + 6



f00TargaLoader::f00TargaLoader()
:	f00ImageLoader(),
	d_imageDepth( 0 )
{
}


f00TargaLoader::~f00TargaLoader()
{
}


bool
f00TargaLoader::loadImage( const char* pData, int lenData )
{
	assert( pData != 0 );
	return( loadHeader( pData, lenData ) && loadBody( pData, lenData ) );
}


bool
f00TargaLoader::loadImageFromFile( const char* pFileName )
{
	if( pFileName == 0 )
		return false;
	f00FileLoader theFile( pFileName);
	if( theFile.length() <= 0 )
		return false;
	return loadImage( theFile.access(), theFile.length() );
}


bool
f00TargaLoader::loadHeader( const char* pData, int maxLen )
{
	resetRGBA();

	d_commentOffset = int( (unsigned char)*pData );

	if( memcmp( pData + 1, DEF_targaHeaderContent, DEF_targaHeaderLength - 1 ) != 0 )
		return false;

	unsigned char smallArray[ 2 ];
	int width;
	int height;

	memcpy( smallArray, pData + DEF_targaHeaderLength + 0, 2 );
	width = smallArray[ 0 ] + smallArray[ 1 ] * 0x0100;

	memcpy( smallArray, pData + DEF_targaHeaderLength + 2, 2 );
	height = smallArray[ 0 ] + smallArray[ 1 ] * 0x0100;

	memcpy( smallArray, pData + DEF_targaHeaderLength + 4, 2 );
	int depth = smallArray[ 0 ];
//	int bitsPerPixelFlag = smallArray[ 1 ];

	// Cannot have negative sizes.
	if( ( width <= 0 ) || ( height <= 0 ) )
		return false;

	// If the image is more than 1 megatexel, you should not be using this class!
	assert( width * height < 1048576 );

	// Only allow 24-bit and 32-bit!  FIXME?
	bool is24Bit( depth == 24 );
	bool is32Bit( depth == 32 );
	if( !( is24Bit || is32Bit ) )
		return false;

	// Is the file too short?
	if( maxLen < ( DEF_targaBodyStartsAt + width * height * depth / 8 ) )
		return false;

	setSize( width, height );
	d_imageDepth = depth;
	return true;
}


bool
f00TargaLoader::loadBody( const char *pData, int maxLen )
{
	int width( getWidth() );
	int height( getHeight() );

	bool is32Bit( d_imageDepth == 32 );
	bool is24Bit( d_imageDepth == 24 );

	int bodySize( width * height * d_imageDepth / 8 );
	int imageSize( width * height * 4 );						// RGBA == 4 bytes.
	assert( bodySize > 0 );

	int bytesRead( DEF_targaBodyStartsAt );
	bytesRead += d_commentOffset;

	GLubyte* pBuffer = new GLubyte[ imageSize ];
	if( is32Bit )
	{
		// Easy, just copy it.
		assert( imageSize == bodySize );
		assert( bodySize <= maxLen );
		memcpy( pBuffer, pData + bytesRead, bodySize );
	}
	else if( is24Bit )
	{
		assert( imageSize <= ( ( imageSize * 4 ) / 3 ) );
		for( int loop = 0; loop < imageSize; loop += 4, bytesRead += 3 )
		{
			memcpy( pBuffer + loop, pData + bytesRead, 3 );
			pBuffer[ loop + 3 ] = 255;			// Force alpha to max?!
		}
	}
	else
		assert( false );		// Someone broke something!

	// Swap R and B bytes.
	GLubyte ubTemp;
	for( int loop = 0; loop < imageSize; loop += 4 )
	{
		ubTemp = pBuffer[ loop + 0 ];
		pBuffer[ loop + 0 ] = pBuffer[ loop + 2 ];
		pBuffer[ loop + 2 ] = ubTemp;
	}

	// Store it.
	setRGBA( pBuffer );

	return true;
}



