
#ifdef __BORLANDC__
	// Borland stuff.
	#include <windows.h>
	#pragma hdrstop
#endif


// Primary include.
#include "f00TextureObject.h"

// System includes.
#include <GL/glu.h>
#include <algorithm>
#include <math.h>
#include <assert.h>



//
//	f00RAMTexture
//

f00RAMTexture::f00RAMTexture( GLubyte* pImageRGBA, int width, int height )
:	d_pImageRGBA( 0 ),
	d_refCount( 0 )
{
	set( pImageRGBA, width, height );
}


f00RAMTexture::f00RAMTexture( const f00RAMTexture& copyFrom )
:	d_width( copyFrom.d_width ),
	d_height( copyFrom.d_height ),
	d_refCount( 0 )
{
	int arraySize( d_width * d_height * 4 );		// RGBA == 4 bytes
	d_pImageRGBA = new GLubyte[ arraySize ];
	memcpy( d_pImageRGBA, copyFrom.d_pImageRGBA, arraySize );
}


f00RAMTexture::~f00RAMTexture()
{
	assert( d_refCount == 0 );
	clear();
}


void
f00RAMTexture::clear()
{
	delete[] d_pImageRGBA;
	d_pImageRGBA = 0;
	d_width = 0;
	d_height = 0;
}


bool
f00RAMTexture::set( GLubyte* pImageRGBA, int width, int height )
{
	assert( pImageRGBA != 0 );
	clear();
	d_pImageRGBA = pImageRGBA;
	d_width = width;
	d_height = height;
	return true;
}



//
//	f00GLTexture
//

GLint
f00GLTexture::d_magFilterDefault = GL_LINEAR;

GLint
f00GLTexture::d_minFilterDefault = GL_LINEAR_MIPMAP_LINEAR;


f00GLTexture::f00GLTexture()
{
	glGenTextures( 1, &d_textureID );
	assert( d_textureID != 0 );
}


f00GLTexture::f00GLTexture( const f00GLTexture& copyFrom )
{
	assert( false );	// Not yet implemented.
}


f00GLTexture::f00GLTexture( f00RAMTexture& rRAMTex, GLint magFilter, GLint minFilter, GLint wrapS, GLint wrapT )
{
	glGenTextures( 1, &d_textureID );
	assert( d_textureID != 0 );
	loadTexture( rRAMTex, magFilter, minFilter, wrapS, wrapT );
}


f00GLTexture::~f00GLTexture()
{
	glDeleteTextures( 1, &d_textureID );
	d_textureID = 0;
}


bool
f00GLTexture::loadTexture( f00RAMTexture& rRAMTex, GLint magFilter, GLint minFilter, GLint wrapS, GLint wrapT )
{
	GLint localMagFilter( ( magFilter == GL_NONE ) ? d_magFilterDefault : magFilter );
	GLint localMinFilter( ( minFilter == GL_NONE ) ? d_minFilterDefault : minFilter );

	// Get our memory object and it's dimensions.
	int width = rRAMTex.width();
	int height = rRAMTex.height();
	GLubyte* pImageRGBA = rRAMTex.getImageRGBA();
	assert( pImageRGBA != 0 );

	// Get the right context, first.
	glBindTexture( GL_TEXTURE_2D, d_textureID );

	// Do some OpenGL prep.
	glPixelStorei( GL_UNPACK_ALIGNMENT, 1 );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, wrapS );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, wrapT );
	glTexEnvi( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE );

	// Apply mip-map 'quality hint' prefs.
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, localMagFilter );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, localMinFilter );

	// Scale up to (at most) 256x256 (or to the lowest OpenGL can take, whichever is smaller....
	GLint glMaxTexDim;
	glGetIntegerv( GL_MAX_TEXTURE_SIZE, &glMaxTexDim );
	glMaxTexDim = std::min( (GLint)256, glMaxTexDim );

	// Determine appropriate power of 2 for X.
	double xPow2, yPow2;
	if( width <= glMaxTexDim )
		xPow2 = log( (double)width ) / log( (double)2.0 );
	else
		xPow2 = log( (double)glMaxTexDim ) / log( (double)2.0 );
	int ixPow2 = (int)xPow2;

	// Determine appropriate power of 2 for Y.
	if( height <= glMaxTexDim )
		yPow2 = log( (double)height ) / log( (double)2.0 );
	else
		yPow2 = log( (double)glMaxTexDim ) / log( (double)2.0 );
	int iyPow2 = (int)yPow2;

	// Scale up to the next whole power.
	if( xPow2 != (double)ixPow2 )
		ixPow2++;
	if( yPow2 != (double)iyPow2 )
		iyPow2++;

	// New sizes.
	int xSize2 = 1 << ixPow2;
	int ySize2 = 1 << iyPow2;

	// Stretch it?
	GLubyte* pData;
	bool isResized( ( width != xSize2 ) || ( height != ySize2 ) );
	if( isResized )
	{
		pData = new unsigned char[ xSize2 * ySize2 * 4 ];		// RGBA == 4 bytes
		gluScaleImage(	GL_RGBA, width, height, GL_UNSIGNED_BYTE, pImageRGBA, xSize2, ySize2, GL_UNSIGNED_BYTE, pData );
	}
	else
		pData = pImageRGBA;
	assert( pData != 0 );

	// If we need MIPMaps, build them with glu, else, load it directly.
	if( isMIPMap( localMinFilter ) )
		gluBuild2DMipmaps( GL_TEXTURE_2D, 4, xSize2, ySize2, GL_RGBA, GL_UNSIGNED_BYTE, pData );
	else
		glTexImage2D( GL_TEXTURE_2D, 0, GL_RGBA, xSize2, ySize2, 0, GL_RGBA, GL_UNSIGNED_BYTE, pData );

	// Cleanup.
	if( isResized )
		delete[] pData;

	return true;
}


bool
f00GLTexture::selectTexture() const
{
	if( d_textureID == 0 )
		return false;
	glBindTexture( GL_TEXTURE_2D, d_textureID );
	return true;
}



f00TextureObject::f00TextureObject( f00RAMTexture* pRAMTexture, GLint magFilter, GLint minFilter, GLint wrapS, GLint wrapT )
:  d_myGLTexture(),
	d_magFilter( magFilter ),
	d_minFilter( minFilter ),
	d_wrapS( wrapS ),
	d_wrapT( wrapT )
{
	assert( pRAMTexture != 0 );
	d_pRAMTexture = pRAMTexture;
	d_pRAMTexture -> incrementRefCount();
}


f00TextureObject::f00TextureObject( const f00TextureObject& copyFrom, bool shareRAMTexture, GLint magFilter, GLint minFilter )
:  d_myGLTexture(),
	d_magFilter( ( magFilter != GL_NONE ) ? magFilter : copyFrom.d_magFilter ),
	d_minFilter( ( minFilter != GL_NONE ) ? minFilter : copyFrom.d_minFilter ),
	d_wrapS( copyFrom.d_wrapS ),
	d_wrapT( copyFrom.d_wrapT )
{
	d_textureID = d_myGLTexture.textureID();

	if( shareRAMTexture )
	{
		assert( copyFrom.d_pRAMTexture != 0 );
		d_pRAMTexture = copyFrom.d_pRAMTexture;
	}
	else
		d_pRAMTexture = new f00RAMTexture( *( copyFrom.d_pRAMTexture ) );

	d_pRAMTexture -> incrementRefCount();

	loadRAMIntoGL();
}


f00TextureObject::f00TextureObject( f00ImageLoader& anImageLoader, bool extractImagePointer, GLint magFilter, GLint minFilter, GLint wrapS, GLint wrapT )
:	d_myGLTexture(),
	d_magFilter( magFilter ),
	d_minFilter( minFilter ),
	d_wrapS( wrapS ),
	d_wrapT( wrapT )
{

	int width;
	int height;
	GLubyte* pImageBuffer;
	if( extractImagePointer )
	{
		pImageBuffer = anImageLoader.extractRGBA( width, height );
	}
	else
	{
		int bufLen( anImageLoader.getHeight() * anImageLoader.getWidth() );
		pImageBuffer = new GLubyte[ bufLen ];
		memcpy( (void*)pImageBuffer, (void*)anImageLoader.getRGBA( width, height ), bufLen );
	}

	d_pRAMTexture = new f00RAMTexture( pImageBuffer, width, height );

	d_pRAMTexture -> incrementRefCount();

	loadRAMIntoGL();
}


f00TextureObject::~f00TextureObject()
{
	if( d_pRAMTexture != 0 )
	{
		if( d_pRAMTexture -> decrementRefCount() == 0 )
			delete d_pRAMTexture;
		d_pRAMTexture = 0;
	}
}


void
f00TextureObject::setTextureQuality( GLint magFilter, GLint minFilter )
{
	d_magFilter = magFilter;
	d_minFilter = minFilter;
}


void
f00TextureObject::setWrapMode( GLint wrapS, GLint wrapT )
{
	d_wrapS = wrapS;
	d_wrapT = wrapT;
}


bool
f00TextureObject::loadRAMIntoGL( GLint magFilter, GLint minFilter )
{
	if( d_pRAMTexture == 0 )
		return false;
	return d_myGLTexture.loadTexture( *d_pRAMTexture, ( magFilter != GL_NONE ) ? magFilter : d_magFilter, ( minFilter != GL_NONE ) ? minFilter : d_minFilter, d_wrapS, d_wrapT );
}


bool
f00TextureObject::select() const
{
	return d_myGLTexture.selectTexture();
}



