
#ifdef __BORLANDC__
	// Borland stuff.
	#include <condefs.h>
	#include <windows.h>
	#pragma hdrstop
#endif


#ifdef _MSC_VER
	#include <windows.h>
#endif


// Primary include.
#include "f00TimeBase.h"


// System-time access mechanism(s).
#ifdef __WIN32__
	//	#define DEF_Use_ QueryPerformanceCounter
	#define DEF_Use_timeGetTime
	#include <mmsystem.h>
	#include <assert.h>
	static UINT minTimePeriod = 0;
#else
	// Default to the standard 'clock()' if nothing better.
	#include <time.h>
#endif



f00TimeBase::f00TimeBase( int frameIntervalInMS )
:	d_initialTimeInMS( 0 ),
	d_currentTimeInMS( 0 ),
	d_frameIntervalInMS( frameIntervalInMS ),
	d_currentFrameBase( -1 ),
	d_currentFrameOffset( 0.0 ),
	d_lastFrameBase( -1 ),
	d_lastFrameOffset( 0.0 ),
	d_lastFrameDelta( 0.0 )
{
	#ifdef DEF_Use_timeGetTime

		TIMECAPS timeCaps;
		assert( timeGetDevCaps( &timeCaps, sizeof( timeCaps ) ) == TIMERR_NOERROR );
		minTimePeriod = timeCaps.wPeriodMin;
		assert( minTimePeriod > 0 );
		assert( timeBeginPeriod( minTimePeriod ) == TIMERR_NOERROR );

	#endif

	d_initialTimeInMS = getSystemMS();
}


f00TimeBase::~f00TimeBase()
{
	#ifdef DEF_Use_timeGetTime

		assert( timeEndPeriod( minTimePeriod ) == TIMERR_NOERROR );

	#endif
}


void
f00TimeBase::updateCurrentMS()
{
	d_currentTimeInMS = ( getSystemMS() - d_initialTimeInMS );
}


void
f00TimeBase::updateFrameValues( bool fetchLatestTime )
{
	if( d_frameIntervalInMS <= 0 )
		return;

	if( fetchLatestTime )
		updateCurrentMS();


	d_currentFrameBase = d_currentTimeInMS / d_frameIntervalInMS;

	int msIntoFrame( d_currentTimeInMS - ( d_currentFrameBase * d_frameIntervalInMS ) );

	d_currentFrameOffset = (float)msIntoFrame / (float)d_frameIntervalInMS;

	if( d_lastFrameBase == -1 )
	{
		d_lastFrameDelta = 0.0;
	}
	else
	{
		int deltaBase( d_currentFrameBase - d_lastFrameBase );
		float deltaOffset( d_currentFrameOffset - d_lastFrameOffset );
		d_lastFrameDelta = deltaBase + deltaOffset;
	}

	d_lastFrameBase = d_currentFrameBase;
	d_lastFrameOffset = d_currentFrameOffset;

}


int
f00TimeBase::getSystemMS()
{
	#ifdef DEF_Use_timeGetTime

		// FIXME  For now, use timeGetTime ... add QueryPerformance* stuff later.
		return timeGetTime();

	#else

		// Cheap & simple optimization for machine with a 1ms clock....
		static float clockScalar = -1.0;
		if( clockScalar == 1.0 )
			return clock();

		if( clockScalar < 0.0 )
			clockScalar = 1000.0 / CLOCKS_PER_SEC;
		return int( clock() * clockScalar );

	#endif
}



