#ifndef f00TimeBaseH
#define f00TimeBaseH


// System includes.
#include <GL/gl.h>


/*

Here, we keep track of the simulation/game/whatever's current time.

Given an optional frame-time-in-ms, we can generate a curren frame base
(integral) and a offset (float, 0.0 to 0.99999...).  Great for smooth
motion/animation/keyframing....

This may also used by a mouse-type interface for smoothing....

*/


class f00TimeBase
{

	public:

		f00TimeBase( int frameIntervalInMS = -1 );

		~f00TimeBase();


	public:

		void
		updateCurrentMS();

		int
		getCurrentMS( bool fetchLatestTime = true )
		{
			if( fetchLatestTime )
				updateCurrentMS();
			return d_currentTimeInMS;
		}


		void
		updateFrameValues( bool fetchLatestTime = true );

		int
		getFrameInterval() const
		{
			return d_frameIntervalInMS;
		}

		int
		getCurrentFrameBase() const
		{
			return d_currentFrameBase;
		}

		float
		getCurrentFrameOffset() const
		{
			return d_currentFrameOffset;
		}

		float
		getLastFrameDelta() const
		{
			return d_lastFrameDelta;
		}


	private:

		int
		getSystemMS();


	private:

		int								d_initialTimeInMS;

		int								d_currentTimeInMS;

		int								d_frameIntervalInMS;
		int								d_currentFrameBase;
		float								d_currentFrameOffset;

		int								d_lastFrameBase;
		float								d_lastFrameOffset;

		float								d_lastFrameDelta;

		
};


#endif

